function [visualisation,u,v]=compute_optic_flow_hs(im1,im2,alpha,num_iterations)
% Use Horn & Schunck's method to compute
% and visualise the optic flow field 
% for image sequence 'im1' and 'im2'.
% Use smoothing term weight 'alpha'
% and run for 'num_iterations' iterations.
% If 'num_iterations' is a vector, then
% run for num_iterations(end) iterations
% and visualise intermediate results 
% corresponding to the vector's entries.
%
% 'visualisation' contains the false color
% representation(s) of the flow field.
% 2D array 'u' contains the vertical flow components
% 2D array 'v' contains the horizontal flow components
%
% Author: Nazar Khan (PUCIT)
% Date: December 2015

if nargin<4
    num_iterations=100;
    if nargin<3
        alpha=1;
    end
end

if ndims(im1)==3
    im1=double(rgb2gray(im1));
    im2=double(rgb2gray(im2));
else
    im1=double(im1);
    im2=double(im2);
end

ds_factor=4;
im1=imresize(im1,1/ds_factor);
im2=imresize(im2,1/ds_factor);

%% normalise images between 0 and 1
im1=myAffineRescaling(im1,1);
im2=myAffineRescaling(im2,1);

%% Kernels for derivative filtering
 % x is the vertically downwards direction
 % y is the horizontal direction from left to right
 % We use 
%mask_x=[1;0;-1]/2;          %central difference approximation of 1st-order derivative
mask_x=[-1;0;1]*[1 2 1];    %vertical Sobel operator
mask_x=rot90(mask_x,2);

%% compute spatial and temporal gradients
fx=myConvolution_ReflectingBoundaries(im1,mask_x);  %vertical
fy=myConvolution_ReflectingBoundaries(im1,mask_x'); %horizontal
fz=im2-im1;                                         %temporal

%% pre-compute gradient data for structure tensor
fxfx=fx.^2;
fyfy=fy.^2;
fxfy=fx.*fy;
fxfz=fx.*fz;
fyfz=fy.*fz;

%initialise flow field to zero vectors
u=zeros(size(im1));
v=zeros(size(im1));
unew=u;
vnew=v;

%helper variables
h=1;
helper1=alpha/h^2;
helper4=helper1*8;

visualisation=[];
skp=1;
for iter=1:num_iterations(end)
    helper2=fxfy.*v+fxfz;
    helper3=fxfy.*u+fyfz;
    for i=2:skp:size(im1,1)-1
        for j=2:skp:size(im1,2)-1
            unew(i,j) = --------ADD_CODE_HERE--------
            vnew(i,j) = --------ADD_CODE_HERE--------
        end
    end
    u=unew;
    v=vnew;
    %% compute false color representation of flow field
    imagesc(visualise_optic_flow(u,v)); axis image; drawnow; %comment this line for computing results faster
    if ismember(iter,num_iterations)
        visualisation = [visualisation visualise_optic_flow(u,v)];
    end
end