function [visualisation,flow_class]=compute_optic_flow_lk(im1,im2,rho)
% Use Lucas & Kanade's method to compute
% and visualise the optic flow field 
% for image sequence 'im1' and 'im2'.
% Use Gaussian smoothing kernel 
% with standard deviation 'rho'.
%
% 'visualisation' contains the false color
% representation of the flow field.
% 'flow_class' contains the classification of
% optic flow vectors as 
% 	0 	=> 	no flow
% 	0.5 	=> 	normal flow
% 	1 	=> 	valid optic flow
%
% Author: Nazar Khan (PUCIT)
% Date: December 2015

if nargin<3
    rho=4;
end

if ndims(im1)==3
    im1=double(rgb2gray(im1));
    im2=double(rgb2gray(im2));
else
    im1=double(im1);
    im2=double(im2);
end

ds_factor=1;
im1=imresize(im1,1/ds_factor);
im2=imresize(im2,1/ds_factor);

%% normalise images between 0 and 1
im1=myAffineRescaling(im1,1);
im2=myAffineRescaling(im2,1);

%% Kernels for derivative filtering
 % x is the vertically downwards direction
 % y is the horizontal direction from left to right
 % We use 
%mask_x=[1;0;-1]/2;          %central difference approximation of 1st-order derivative
mask_x=[-1;0;1]*[1 2 1];    %vertical Sobel operator
mask_x=rot90(mask_x,2);

%% compute spatial and temporal gradients
fx=myConvolution_ReflectingBoundaries(im1,mask_x);  %vertical
fy=myConvolution_ReflectingBoundaries(im1,mask_x'); %horizontal
fz=im2-im1;                                         %temporal

%% pre-compute gradient data for structure tensor
fxfx=fx.^2;
fyfy=fy.^2;
fxfy=fx.*fy;
fxfz=fx.*fz;
fyfz=fy.*fz;

gMask=gaussianMask(rho);
fxfxs=separable_convolve(fxfx,gMask);
fyfys=separable_convolve(fyfy,gMask);
fxfys=separable_convolve(fxfy,gMask);
fxfzs=separable_convolve(fxfz,gMask);
fyfzs=separable_convolve(fyfz,gMask);
spatial_gradient_magnitude=fxfxs+fyfys;

%% compute normal flow
un=--------ADD_CODE_HERE--------
vn=--------ADD_CODE_HERE--------
normal_flow=cat(3,un,vn);
normal_flow_magnitude=sum(normal_flow.^2,3);

%% compute flow at every pixel and classify into no flow, normal flow, optic flow
skp=1;
optic_flow=zeros([size(im1) 2]);
flow_class=zeros(size(im1));
for i=1:skp:size(im1,1)
    for j=1:skp:size(im1,2)
	   %set up the 2x2 stucture tensor J
        J=--------ADD_CODE_HERE--------

        if trace(J)<5e-3		%rank(J)=0
            flow_class(i,j)=0;  %no spatial gradients. no flow
        elseif det(J)<5e-6		%rank(J)=1
            flow_class(i,j)=.5; %uniform spatial gradients. normal flow
            optic_flow(i,j,:)=normal_flow(i,j,:);
        else
            flow_class(i,j)=1;  %varying spatial gradients. reliable optic flow
            b=-[fxfzs(i,j) ; fyfzs(i,j)];
            optic_flow(i,j,:)=--------ADD_CODE_HERE--------
        end
    end
end

%% compute false color representation of flow field
visualisation = visualise_optic_flow(optic_flow(:,:,1),optic_flow(:,:,2));

% imagesc(im1);
% qdata=[];
% for i=1:size(im1,1)
%     for j=1:size(im1,2)
%         qdata=[qdata;[i,j,optic_flow(i,j,1),optic_flow(i,j,2)]];
%     end
% end
% hold on;
% sk=1;
% quiver(qdata(1:sk:end,2),qdata(1:sk:end,1),qdata(1:sk:end,4),qdata(1:sk:end,3),1,'b','LineWidth',1);