function C=myConvolution_ReflectingBoundaries(I,M)
% Usage: C=myConvolution_ReflectingBoundaries(I,M)
% For convolving a 2D grayscale image I with a convolution mask M.
% Reflecting boundaries are used so that convolved image is the same size
% as the original image AND boundaries are handled in a principled manner.
% Mask M can be 2D as well as 1D.
% Author: Nazar Khan (PUCIT)
% Date: December 2014

%convert to double since convolution can involve fractions
I=double(I);
M=double(M);

%Mask needs to be inverted in both directions to perform convolution.
%This is required because of the mathematical definition of convolution.
M=fliplr(flipud(M));

[mrows,mcols]=size(M);      %mask size
border_rows=floor(mrows/2); %number of rows to skip from border
border_cols=floor(mcols/2); %number of columns to skip from border

%% Set up mirrored boundaries
%make larger image
I1=zeros(size(I,1)+2*border_rows,size(I,2)+2*border_cols);
[irows,icols]=size(I1);      %image size
%copy image into larger image
I1(border_rows+1:irows-border_rows,border_cols+1:icols-border_cols)=I;
%mirror top border rows and copy into larger image
for i=1:border_rows
    I1(i,border_cols+1:icols-border_cols)=I1(mrows-i+1,border_cols+1:icols-border_cols);
end
%mirror bottom border rows and copy into larger image
for i=1:border_rows
    I1(irows-border_rows+i,border_cols+1:icols-border_cols)=I1(irows-border_rows-i,border_cols+1:icols-border_cols);
end
%mirror left border cols and copy into larger image
for i=1:border_cols
    I1(border_rows+1:irows-border_rows,i)=I1(border_rows+1:irows-border_rows,mcols-i+1);
end
%mirror right border cols and copy into larger image
for i=1:border_cols
    I1(border_rows+1:irows-border_rows,icols-border_cols+i)=I1(border_rows+1:irows-border_rows,icols-border_cols-i);
end
I=I1;
clear I1;

%convolution loop
C=zeros(irows,icols);       %convolved image
for i=1+border_rows:irows-border_rows
    for j=1+border_cols:icols-border_cols
        image_patch_under_mask=...
            I(i-border_rows:i+border_rows,j-border_cols:j+border_cols);
        C(i,j)=sum(sum(M.*image_patch_under_mask));
    end
end

%ignore the artificially added boundaries from the convolved image so that
%it is the same size as the input image
C=C(border_rows+1:irows-border_rows,border_cols+1:icols-border_cols);